==> geant4.10.05.tar.gz: geant4.10.05/examples/extended/parallel/ThreadsafeScorers/ts_scorers.cc <==
//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file parallel/ThreadsafeScorers/ts_scorers.cc
/// \brief Main of the ThreadsafeScorers example
//
//
//
//
/// ts_scorers example shows how to use global scorers. The benefit of using
///     global scorers in memory-savings for problems with very large amounts
///     of scoring volumes. Additionally, the global scorers are more precise
///     w.r.t. the serial solution because of the lack of compounding
///     round-off error from multiple threads
///
/// In this example, the global scorers are implemented as static member
///     variables in TSRun because TSRun is thread-local. The G4atomic
///     class is the core of the thread-safe scorers and can be uses
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4Types.hh"

#ifdef G4MULTITHREADED
    #include "G4MTRunManager.hh"
    #include "G4Threading.hh"
    typedef G4MTRunManager RunManager;
#else
    #include "G4RunManager.hh"
    typedef G4RunManager RunManager;
#endif

#include "Randomize.hh"

// User Defined Classes
#include "TSDetectorConstruction.hh"
#include "TSPhysicsList.hh"
#include "TSActionInitialization.hh"

#include "G4UImanager.hh"
#include "G4VisExecutive.hh"
#include "G4UIExecutive.hh"
#include "G4TiMemory.hh"

// for std::system(const char*)
#include <cstdlib>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void message(RunManager* runmanager)
{
#ifdef G4MULTITHREADED
    runmanager->SetNumberOfThreads(G4Threading::G4GetNumberOfCores());
    G4cout << "\n\n\t--> Running in multithreaded mode with "
           << runmanager->GetNumberOfThreads()
           << " threads\n\n" << G4endl;
#else
    // get rid of unused variable warning
    runmanager->SetVerboseLevel(runmanager->GetVerboseLevel());
    G4cout << "\n\n\t--> Running in serial mode\n\n" << G4endl;
#endif
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

int main(int argc, char** argv)
{

    // Detect interactive mode (if no arguments) and define UI session
    //
    G4UIExecutive* ui = 0;
    if(argc == 1)
        ui = new G4UIExecutive(argc, argv);

    // Set the random seed
    CLHEP::HepRandom::setTheSeed(1245214UL);

    RunManager* runmanager = new RunManager();

    message(runmanager);

    runmanager->SetUserInitialization(new TSDetectorConstruction);

    runmanager->SetUserInitialization(new TSPhysicsList);

    runmanager->SetUserInitialization(new TSActionInitialization);

    runmanager->Initialize();


    // Initialize visualization
    //
    G4VisManager* visManager = new G4VisExecutive;
    // G4VisExecutive can take a verbosity argument - see /vis/verbose guidance.
    // G4VisManager* visManager = new G4VisExecutive("Quiet");
    visManager->Initialize();

    // Get the pointer to the User Interface manager
    G4UImanager* UImanager = G4UImanager::GetUIpointer();

    // Process macro or start UI session
    //
    if (!ui)
    {
        // batch mode
        G4String command = "/control/execute ";
        G4String fileName = argv[argc-1];
        UImanager->ApplyCommand(command+fileName);
    } else
    {
        // interactive mode
        UImanager->ApplyCommand("/control/execute vis.mac");
        ui->SessionStart();
        delete ui;
    }

#ifdef GEANT4_USE_TIMEMORY
    G4cout << "\nOutputting TiMemory results...\n" << G4endl;
    tim::manager* manager = tim::manager::instance();
#   if defined(G4MULTITHREADED)
    // TiMemory by default will report master + workers
    manager->set_get_num_threads_func(
                [=]() { return runmanager->GetNumberOfThreads(); });
#   endif
    manager->write_report(std::cout, true);
    std::string fname = argv[0];
    std::string rfname = fname + ".txt";
    std::string sfname = fname + ".json";
    manager->write_report(rfname);
    manager->write_serialization(sfname);
    if(std::system(nullptr))
    {
        std::stringstream ss;
        ss << "timemory-plotter -f " << sfname << " -t \"ThreadSafe Scorers\" "
           << "-o plots -e --timing-fields cpu wall sys";
        int ret = std::system(ss.str().c_str());
        if(ret != 0)
        {
            G4ExceptionDescription desc;
            desc << "Error generating plots with command: '"
                 << ss.str() << "'";
            G4Exception("ThreadsafeScorers::main", "0001", JustWarning, desc);
        }
    }
#endif

    // Job termination
    // Free the store: user actions, physics_list and detector_description are
    // owned and deleted by the run manager, so they should not be deleted
    // in the main() program !
    delete visManager;
    delete runmanager;

    return 0;
}

==> geant4.10.06.p03.tar.gz: geant4.10.06.p03/examples/extended/parallel/ThreadsafeScorers/ts_scorers.cc <==
//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file parallel/ThreadsafeScorers/ts_scorers.cc
/// \brief Main of the ThreadsafeScorers example
//
//
//
//
/// ts_scorers example shows how to use global scorers. The benefit of using
///     global scorers in memory-savings for problems with very large amounts
///     of scoring volumes. Additionally, the global scorers are more precise
///     w.r.t. the serial solution because of the lack of compounding
///     round-off error from multiple threads
///
/// In this example, the global scorers are implemented as static member
///     variables in TSRun because TSRun is thread-local. The G4atomic
///     class is the core of the thread-safe scorers and can be uses
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4Types.hh"

#ifdef G4MULTITHREADED
    #include "G4MTRunManager.hh"
    #include "G4Threading.hh"
    typedef G4MTRunManager RunManager;
#else
    #include "G4RunManager.hh"
    typedef G4RunManager RunManager;
#endif

#include "Randomize.hh"

// User Defined Classes
#include "TSDetectorConstruction.hh"
#include "TSPhysicsList.hh"
#include "TSActionInitialization.hh"

#include "G4UImanager.hh"
#include "G4VisExecutive.hh"
#include "G4UIExecutive.hh"
#include "G4TiMemory.hh"

// for std::system(const char*)
#include <cstdlib>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void message(RunManager* runmanager)
{
#ifdef G4MULTITHREADED
    runmanager->SetNumberOfThreads(G4Threading::G4GetNumberOfCores());
    G4cout << "\n\n\t--> Running in multithreaded mode with "
           << runmanager->GetNumberOfThreads()
           << " threads\n\n" << G4endl;
#else
    // get rid of unused variable warning
    runmanager->SetVerboseLevel(runmanager->GetVerboseLevel());
    G4cout << "\n\n\t--> Running in serial mode\n\n" << G4endl;
#endif
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

int main(int argc, char** argv)
{
    TIMEMORY_INIT(argc, argv);

#if defined(GEANT4_USE_TIMEMORY)
    // override environment settings
    tim::settings::json_output() = true;
    tim::settings::dart_output() = true;
    tim::settings::dart_type() = "peak_rss";
    tim::settings::dart_count() = 1;
#endif

    // Detect interactive mode (if no arguments) and define UI session
    //
    G4UIExecutive* ui = 0;
    if(argc == 1)
        ui = new G4UIExecutive(argc, argv);

    // Set the random seed
    CLHEP::HepRandom::setTheSeed(1245214UL);

    RunManager* runmanager = new RunManager();

    message(runmanager);

    runmanager->SetUserInitialization(new TSDetectorConstruction);

    runmanager->SetUserInitialization(new TSPhysicsList);

    runmanager->SetUserInitialization(new TSActionInitialization);

    runmanager->Initialize();


    // Initialize visualization
    //
    G4VisManager* visManager = new G4VisExecutive;
    // G4VisExecutive can take a verbosity argument - see /vis/verbose guidance.
    // G4VisManager* visManager = new G4VisExecutive("Quiet");
    visManager->Initialize();

    // Get the pointer to the User Interface manager
    G4UImanager* UImanager = G4UImanager::GetUIpointer();

    // Process macro or start UI session
    //
    if (!ui)
    {
        // batch mode
        G4String command = "/control/execute ";
        G4String fileName = argv[argc-1];
        UImanager->ApplyCommand(command+fileName);
    } else
    {
        // interactive mode
        UImanager->ApplyCommand("/control/execute vis.mac");
        ui->SessionStart();
        delete ui;
    }

    // Job termination
    // Free the store: user actions, physics_list and detector_description are
    // owned and deleted by the run manager, so they should not be deleted
    // in the main() program !
    delete visManager;
    delete runmanager;

    return 0;
}
